@@ -45,11 +45,12 @@
 #include <named/config.h>
 #include <named/globals.h>
 
-#include "bind.keys.h"
+#include <bind.keys.h>
 
 /*% default configuration */
 static char defaultconf[] = "\
 options {\n\
+	automatic-interface-scan yes;\n\
 #	blackhole {none;};\n"
 #ifndef WIN32
 "	coresize default;\n\
@@ -69,7 +70,7 @@ options {\n\
 	host-statistics no;\n\
 	interface-interval 60;\n\
 	listen-on {any;};\n\
-	listen-on-v6 {none;};\n\
+	listen-on-v6 {any;};\n\
 	match-mapped-addresses no;\n\
 	max-rsa-exponent-size 0; /* no limit */\n\
 	memstatistics-file \"named.memstats\";\n\
@@ -78,6 +79,7 @@ options {\n\
 #	pid-file \"" NS_LOCALSTATEDIR "/run/named/named.pid\"; /* or /lwresd.pid */\n\
 	bindkeys-file \"" NS_SYSCONFDIR "/bind.keys\";\n\
 	port 53;\n\
+	prefetch 2 9;\n\
 	recursing-file \"named.recursing\";\n\
 	secroots-file \"named.secroots\";\n\
 "
@@ -108,6 +110,14 @@ options {\n\
 	use-ixfr true;\n\
 	edns-udp-size 4096;\n\
 	max-udp-size 4096;\n\
+"
+#ifdef ISC_PLATFORM_USESIT
+"\
+	nosit-udp-size 4096;\n\
+	request-sit true;\n\
+"
+#endif
+"\
 	request-nsid false;\n\
 	reserved-sockets 512;\n\
 \n\
@@ -164,8 +174,9 @@ options {\n\
 	nsec3-test-zone no;\n\
 	allow-new-zones no;\n\
 "
-#ifdef ALLOW_FILTER_AAAA_ON_V4
+#ifdef ALLOW_FILTER_AAAA
 "	filter-aaaa-on-v4 no;\n\
+	filter-aaaa-on-v6 no;\n\
 	filter-aaaa { any; };\n\
 "
 #endif
@@ -227,17 +238,15 @@ view \"_bind\" chaos {\n\
 	recursion no;\n\
 	notify no;\n\
 	allow-new-zones no;\n\
-"
-#ifdef USE_RRL
-"	# Prevent use of this zone in DNS amplified reflection DoS attacks\n\
+\n\
+	# Prevent use of this zone in DNS amplified reflection DoS attacks\n\
 	rate-limit {\n\
 		responses-per-second 3;\n\
 		slip 0;\n\
 		min-table-size 10;\n\
 	};\n\
-"
-#endif /* USE_RRL */
-"	zone \"version.bind\" chaos {\n\
+\n\
+	zone \"version.bind\" chaos {\n\
 		type master;\n\
 		database \"_builtin version\";\n\
 	};\n\
@@ -281,7 +290,9 @@ ns_config_parsedefaults(cfg_parser_t *parser, cfg_obj_t **conf) {
 }
 
 isc_result_t
-ns_config_get(const cfg_obj_t **maps, const char *name, const cfg_obj_t **obj) {
+ns_config_get(cfg_obj_t const * const *maps, const char *name,
+	      const cfg_obj_t **obj)
+{
 	int i;
 
 	for (i = 0;; i++) {
@@ -404,17 +415,20 @@ ns_config_getzonetype(const cfg_obj_t *zonetypeobj) {
 isc_result_t
 ns_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 		    in_port_t defport, isc_mem_t *mctx,
-		    isc_sockaddr_t **addrsp, isc_uint32_t *countp)
+		    isc_sockaddr_t **addrsp, isc_dscp_t **dscpsp,
+		    isc_uint32_t *countp)
 {
 	int count, i = 0;
 	const cfg_obj_t *addrlist;
-	const cfg_obj_t *portobj;
+	const cfg_obj_t *portobj, *dscpobj;
 	const cfg_listelt_t *element;
 	isc_sockaddr_t *addrs;
 	in_port_t port;
+	isc_dscp_t dscp = -1, *dscps = NULL;
 	isc_result_t result;
 
 	INSIST(addrsp != NULL && *addrsp == NULL);
+	INSIST(dscpsp == NULL || *dscpsp == NULL);
 	INSIST(countp != NULL);
 
 	addrlist = cfg_tuple_get(list, "addresses");
@@ -437,6 +451,23 @@ ns_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 			return (result);
 	}
 
+	if (dscpsp != NULL) {
+		dscps = isc_mem_get(mctx, count * sizeof(isc_dscp_t));
+		if (dscps == NULL)
+			return (ISC_R_NOMEMORY);
+
+		dscpobj = cfg_tuple_get(list, "dscp");
+		if (dscpobj != NULL && cfg_obj_isuint32(dscpobj)) {
+			if (cfg_obj_asuint32(dscpobj) > 63) {
+				cfg_obj_log(dscpobj, ns_g_lctx, ISC_LOG_ERROR,
+					    "dscp value '%u' is out of range",
+					    cfg_obj_asuint32(dscpobj));
+				return (ISC_R_RANGE);
+			}
+			dscp = (isc_dscp_t)cfg_obj_asuint32(dscpobj);
+		}
+	}
+
 	addrs = isc_mem_get(mctx, count * sizeof(isc_sockaddr_t));
 	if (addrs == NULL)
 		return (ISC_R_NOMEMORY);
@@ -445,8 +476,17 @@ ns_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 	     element != NULL;
 	     element = cfg_list_next(element), i++)
 	{
+		const cfg_obj_t *addr;
 		INSIST(i < count);
-		addrs[i] = *cfg_obj_assockaddr(cfg_listelt_value(element));
+		addr = cfg_listelt_value(element);
+		addrs[i] = *cfg_obj_assockaddr(addr);
+		if (dscpsp != NULL) {
+			isc_dscp_t innerdscp;
+			innerdscp = cfg_obj_getdscp(addr);
+			if (innerdscp == -1)
+				innerdscp = dscp;
+			dscps[i] = innerdscp;
+		}
 		if (isc_sockaddr_getport(&addrs[i]) == 0)
 			isc_sockaddr_setport(&addrs[i], port);
 	}
@@ -455,17 +495,26 @@ ns_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 	*addrsp = addrs;
 	*countp = count;
 
+	if (dscpsp != NULL)
+		*dscpsp = dscps;
+
 	return (ISC_R_SUCCESS);
 }
 
 void
 ns_config_putiplist(isc_mem_t *mctx, isc_sockaddr_t **addrsp,
-		    isc_uint32_t count)
+		    isc_dscp_t **dscpsp, isc_uint32_t count)
 {
 	INSIST(addrsp != NULL && *addrsp != NULL);
+	INSIST(dscpsp == NULL || *dscpsp != NULL);
 
 	isc_mem_put(mctx, *addrsp, count * sizeof(isc_sockaddr_t));
 	*addrsp = NULL;
+
+	if (dscpsp != NULL) {
+		isc_mem_put(mctx, *dscpsp, count * sizeof(isc_dscp_t));
+		*dscpsp = NULL;
+	}
 }
 
 static isc_result_t
@@ -499,32 +548,40 @@ get_masters_def(const cfg_obj_t *cctx, const char *name,
 isc_result_t
 ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 			  isc_mem_t *mctx, isc_sockaddr_t **addrsp,
-			  dns_name_t ***keysp, isc_uint32_t *countp)
+			  isc_dscp_t **dscpsp, dns_name_t ***keysp,
+			  isc_uint32_t *countp)
 {
-	isc_uint32_t addrcount = 0, keycount = 0, i = 0;
+	isc_uint32_t addrcount = 0, dscpcount = 0, keycount = 0, i = 0;
 	isc_uint32_t listcount = 0, l = 0, j;
 	isc_uint32_t stackcount = 0, pushed = 0;
 	isc_result_t result;
 	const cfg_listelt_t *element;
 	const cfg_obj_t *addrlist;
 	const cfg_obj_t *portobj;
+	const cfg_obj_t *dscpobj;
 	in_port_t port;
+	isc_dscp_t dscp;
 	dns_fixedname_t fname;
 	isc_sockaddr_t *addrs = NULL;
+	isc_dscp_t *dscps = NULL;
 	dns_name_t **keys = NULL;
 	struct { const char *name; } *lists = NULL;
 	struct {
 		const cfg_listelt_t *element;
 		in_port_t port;
+		isc_dscp_t dscp;
 	} *stack = NULL;
 
 	REQUIRE(addrsp != NULL && *addrsp == NULL);
+	REQUIRE(dscpsp != NULL && *dscpsp == NULL);
 	REQUIRE(keysp != NULL && *keysp == NULL);
 	REQUIRE(countp != NULL);
 
  newlist:
 	addrlist = cfg_tuple_get(list, "addresses");
 	portobj = cfg_tuple_get(list, "port");
+	dscpobj = cfg_tuple_get(list, "dscp");
+
 	if (cfg_obj_isuint32(portobj)) {
 		isc_uint32_t val = cfg_obj_asuint32(portobj);
 		if (val > ISC_UINT16_MAX) {
@@ -540,6 +597,20 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 			goto cleanup;
 	}
 
+	if (dscpobj != NULL && cfg_obj_isuint32(dscpobj)) {
+		if (cfg_obj_asuint32(dscpobj) > 63) {
+			cfg_obj_log(dscpobj, ns_g_lctx, ISC_LOG_ERROR,
+				    "dscp value '%u' is out of range",
+				    cfg_obj_asuint32(dscpobj));
+			return (ISC_R_RANGE);
+		}
+		dscp = (isc_dscp_t)cfg_obj_asuint32(dscpobj);
+	} else {
+		result = ns_config_getdscp(config, &dscp);
+		if (result != ISC_R_SUCCESS)
+			goto cleanup;
+	}
+
 	result = ISC_R_NOMEMORY;
 
 	element = cfg_list_first(addrlist);
@@ -620,6 +691,7 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 			 */
 			stack[pushed].element = cfg_list_next(element);
 			stack[pushed].port = port;
+			stack[pushed].dscp = dscp;
 			pushed++;
 			goto newlist;
 		}
@@ -641,6 +713,18 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 			addrs = new;
 			addrcount = newlen;
 
+			newsize = newlen * sizeof(isc_dscp_t);
+			oldsize = dscpcount * sizeof(isc_dscp_t);
+			new = isc_mem_get(mctx, newsize);
+			if (new == NULL)
+				goto cleanup;
+			if (dscpcount != 0) {
+				memmove(new, dscps, oldsize);
+				isc_mem_put(mctx, dscps, oldsize);
+			}
+			dscps = new;
+			dscpcount = newlen;
+
 			newsize = newlen * sizeof(dns_name_t *);
 			oldsize = keycount * sizeof(dns_name_t *);
 			new = isc_mem_get(mctx, newsize);
@@ -657,6 +741,9 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 		addrs[i] = *cfg_obj_assockaddr(addr);
 		if (isc_sockaddr_getport(&addrs[i]) == 0)
 			isc_sockaddr_setport(&addrs[i], port);
+		dscps[i] = cfg_obj_getdscp(addr);
+		if (dscps[i] == -1)
+			dscps[i] = dscp;
 		keys[i] = NULL;
 		i++;	/* Increment here so that cleanup on error works. */
 		if (!cfg_obj_isstring(key))
@@ -683,6 +770,7 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 		pushed--;
 		element = stack[pushed].element;
 		port = stack[pushed].port;
+		dscp = stack[pushed].dscp;
 		goto resume;
 	}
 	if (i < addrcount) {
@@ -702,6 +790,19 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 		addrs = new;
 		addrcount = i;
 
+		newsize = i * sizeof(isc_dscp_t);
+		oldsize = dscpcount * sizeof(isc_dscp_t);
+		if (i != 0) {
+			new = isc_mem_get(mctx, newsize);
+			if (new == NULL)
+				goto cleanup;
+			memmove(new, dscps, newsize);
+		} else
+			new = NULL;
+		isc_mem_put(mctx, dscps, oldsize);
+		dscps = new;
+		dscpcount = i;
+
 		newsize = i * sizeof(dns_name_t *);
 		oldsize = keycount * sizeof(dns_name_t *);
 		if (i != 0) {
@@ -724,6 +825,7 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 	INSIST(keycount == addrcount);
 
 	*addrsp = addrs;
+	*dscpsp = dscps;
 	*keysp = keys;
 	*countp = addrcount;
 
@@ -732,6 +834,8 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
  cleanup:
 	if (addrs != NULL)
 		isc_mem_put(mctx, addrs, addrcount * sizeof(isc_sockaddr_t));
+	if (dscps != NULL)
+		isc_mem_put(mctx, dscps, dscpcount * sizeof(isc_dscp_t));
 	if (keys != NULL) {
 		for (j = 0; j < i; j++) {
 			if (keys[j] == NULL)
@@ -751,14 +855,21 @@ ns_config_getipandkeylist(const cfg_obj_t *config, const cfg_obj_t *list,
 
 void
 ns_config_putipandkeylist(isc_mem_t *mctx, isc_sockaddr_t **addrsp,
-			  dns_name_t ***keysp, isc_uint32_t count)
+			  isc_dscp_t **dscpsp, dns_name_t ***keysp,
+			  isc_uint32_t count)
 {
 	unsigned int i;
-	dns_name_t **keys = *keysp;
+	dns_name_t **keys;
 
-	INSIST(addrsp != NULL && *addrsp != NULL);
+	REQUIRE(addrsp != NULL && *addrsp != NULL);
+	REQUIRE(dscpsp == NULL || *dscpsp != NULL);
+	REQUIRE(keysp != NULL && *keysp != NULL);
+
+	keys = *keysp;
 
 	isc_mem_put(mctx, *addrsp, count * sizeof(isc_sockaddr_t));
+	if (dscpsp != NULL)
+		isc_mem_put(mctx, *dscpsp, count * sizeof(isc_dscp_t));
 	for (i = 0; i < count; i++) {
 		if (keys[i] == NULL)
 			continue;
@@ -768,6 +879,8 @@ ns_config_putipandkeylist(isc_mem_t *mctx, isc_sockaddr_t **addrsp,
 	}
 	isc_mem_put(mctx, *keysp, count * sizeof(dns_name_t *));
 	*addrsp = NULL;
+	if (dscpsp != NULL)
+		*dscpsp = NULL;
 	*keysp = NULL;
 }
 
@@ -798,6 +911,31 @@ ns_config_getport(const cfg_obj_t *config, in_port_t *portp) {
 	return (ISC_R_SUCCESS);
 }
 
+isc_result_t
+ns_config_getdscp(const cfg_obj_t *config, isc_dscp_t *dscpp) {
+	const cfg_obj_t *options = NULL;
+	const cfg_obj_t *dscpobj = NULL;
+	isc_result_t result;
+
+	(void)cfg_map_get(config, "options", &options);
+	if (options == NULL)
+		return (ISC_R_SUCCESS);
+
+	result = cfg_map_get(options, "dscp", &dscpobj);
+	if (result != ISC_R_SUCCESS || dscpobj == NULL) {
+		*dscpp = -1;
+		return (ISC_R_SUCCESS);
+	}
+	if (cfg_obj_asuint32(dscpobj) >= 64) {
+		cfg_obj_log(dscpobj, ns_g_lctx, ISC_LOG_ERROR,
+			    "dscp '%u' out of range",
+			    cfg_obj_asuint32(dscpobj));
+		return (ISC_R_RANGE);
+	}
+	*dscpp = (isc_dscp_t)cfg_obj_asuint32(dscpobj);
+	return (ISC_R_SUCCESS);
+}
+
 struct keyalgorithms {
 	const char *str;
 	enum { hmacnone, hmacmd5, hmacsha1, hmacsha224,
