@@ -1,5 +1,6 @@
 /*
- * Portions Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (C) 2004-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (C) 1999-2002  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -13,10 +14,7 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
  * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * See the COPYRIGHT file distributed with this work for additional
- * information regarding copyright ownership.
- *
- * Portions Copyright (C) Network Associates, Inc.
+ * Portions Copyright (C) 1995-2000 by Network Associates, Inc.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -33,20 +31,20 @@
 
 /*%
  * Principal Author: Brian Wellington
+ * $Id: dst_parse.c,v 1.29 2011/08/18 23:46:35 tbox Exp $
  */
 
 #include <config.h>
 
 #include <isc/base64.h>
 #include <isc/dir.h>
-#include <isc/file.h>
 #include <isc/fsaccess.h>
 #include <isc/lex.h>
 #include <isc/mem.h>
-#include <isc/print.h>
 #include <isc/stdtime.h>
 #include <isc/string.h>
 #include <isc/util.h>
+#include <isc/file.h>
 
 #include <dns/time.h>
 #include <dns/log.h>
@@ -95,7 +93,6 @@ static struct parse_map map[] = {
 	{TAG_RSA_COEFFICIENT, "Coefficient:"},
 	{TAG_RSA_ENGINE, "Engine:" },
 	{TAG_RSA_LABEL, "Label:" },
-	{TAG_RSA_PIN, "PIN:" },
 
 	{TAG_DH_PRIME, "Prime(p):"},
 	{TAG_DH_GENERATOR, "Generator(g):"},
@@ -109,14 +106,11 @@ static struct parse_map map[] = {
 	{TAG_DSA_PUBLIC, "Public_value(y):"},
 
 	{TAG_GOST_PRIVASN1, "GostAsn1:"},
+	{TAG_GOST_PRIVRAW, "PrivateKey:"},
 
 	{TAG_ECDSA_PRIVATEKEY, "PrivateKey:"},
-
-	{TAG_EDDSA_PRIVATEKEY, "PrivateKey:"},
-
-	{TAG_EDDSA_PRIVATEKEY, "PrivateKey:"},
-	{TAG_EDDSA_ENGINE, "Engine:" },
-	{TAG_EDDSA_LABEL, "Label:" },
+	{TAG_ECDSA_ENGINE, "Engine:" },
+	{TAG_ECDSA_LABEL, "Label:" },
 
 	{TAG_HMACMD5_KEY, "Key:"},
 	{TAG_HMACMD5_BITS, "Bits:"},
@@ -207,7 +201,9 @@ check_rsa(const dst_private_t *priv, isc_boolean_t external) {
 		have[i] = ISC_TRUE;
 	}
 
-	mask = (1ULL << TAG_SHIFT) - 1;
+	mask = ~0;
+	mask <<= sizeof(mask) * 8 - TAG_SHIFT;
+	mask >>= sizeof(mask) * 8 - TAG_SHIFT;
 
 	if (have[TAG_RSA_ENGINE & mask])
 		ok = have[TAG_RSA_MODULUS & mask] &&
@@ -268,51 +264,41 @@ check_gost(const dst_private_t *priv, isc_boolean_t external) {
 
 	if (priv->nelements != GOST_NTAGS)
 		return (-1);
-	if (priv->elements[0].tag != TAG(DST_ALG_ECCGOST, 0))
+	if ((priv->elements[0].tag != TAG(DST_ALG_ECCGOST, 0)) &&
+	    (priv->elements[0].tag != TAG(DST_ALG_ECCGOST, 1)))
 		return (-1);
 	return (0);
 }
 
 static int
 check_ecdsa(const dst_private_t *priv, isc_boolean_t external) {
-
-	if (external)
-		return ((priv->nelements == 0) ? 0 : -1);
-
-	if (priv->nelements != ECDSA_NTAGS)
-		return (-1);
-	if (priv->elements[0].tag != TAG(DST_ALG_ECDSA256, 0))
-		return (-1);
-	return (0);
-}
-
-static int
-check_eddsa(const dst_private_t *priv, isc_boolean_t external) {
 	int i, j;
-	isc_boolean_t have[EDDSA_NTAGS];
+	isc_boolean_t have[ECDSA_NTAGS];
 	isc_boolean_t ok;
 	unsigned int mask;
 
 	if (external)
 		return ((priv->nelements == 0) ? 0 : -1);
 
-	for (i = 0; i < EDDSA_NTAGS; i++)
+	for (i = 0; i < ECDSA_NTAGS; i++)
 		have[i] = ISC_FALSE;
 	for (j = 0; j < priv->nelements; j++) {
-		for (i = 0; i < EDDSA_NTAGS; i++)
-			if (priv->elements[j].tag == TAG(DST_ALG_ED25519, i))
+		for (i = 0; i < ECDSA_NTAGS; i++)
+			if (priv->elements[j].tag == TAG(DST_ALG_ECDSA256, i))
 				break;
-		if (i == EDDSA_NTAGS)
+		if (i == ECDSA_NTAGS)
 			return (-1);
 		have[i] = ISC_TRUE;
 	}
 
-	mask = (1ULL << TAG_SHIFT) - 1;
+	mask = ~0;
+	mask <<= sizeof(mask) * 8 - TAG_SHIFT;
+	mask >>= sizeof(mask) * 8 - TAG_SHIFT;
 
-	if (have[TAG_EDDSA_ENGINE & mask])
-		ok = have[TAG_EDDSA_LABEL & mask];
+	if (have[TAG_ECDSA_ENGINE & mask])
+		ok = have[TAG_ECDSA_LABEL & mask];
 	else
-		ok = have[TAG_EDDSA_PRIVATEKEY & mask];
+		ok = have[TAG_ECDSA_PRIVATEKEY & mask];
 	return (ok ? 0 : -1 );
 }
 
@@ -382,9 +368,6 @@ check_data(const dst_private_t *priv, const unsigned int alg,
 	case DST_ALG_ECDSA256:
 	case DST_ALG_ECDSA384:
 		return (check_ecdsa(priv, external));
-	case DST_ALG_ED25519:
-	case DST_ALG_ED448:
-		return (check_eddsa(priv, external));
 	case DST_ALG_HMACMD5:
 		return (check_hmac_md5(priv, old));
 	case DST_ALG_HMACSHA1:
@@ -683,7 +666,7 @@ dst__privstruct_writefile(const dst_key_t *key, const dst_private_t *priv,
 	/* XXXDCL return value should be checked for full filesystem */
 	fprintf(fp, "%s v%d.%d\n", PRIVATE_KEY_STR, major, minor);
 
-	fprintf(fp, "%s %u ", ALGORITHM_STR, dst_key_alg(key));
+	fprintf(fp, "%s %d ", ALGORITHM_STR, dst_key_alg(key));
 
 	/* XXXVIX this switch statement is too sparse to gen a jump table. */
 	switch (dst_key_alg(key)) {
@@ -720,12 +703,6 @@ dst__privstruct_writefile(const dst_key_t *key, const dst_private_t *priv,
 	case DST_ALG_ECDSA384:
 		fprintf(fp, "(ECDSAP384SHA384)\n");
 		break;
-	case DST_ALG_ED25519:
-		fprintf(fp, "(ED25519)\n");
-		break;
-	case DST_ALG_ED448:
-		fprintf(fp, "(ED448)\n");
-		break;
 	case DST_ALG_HMACMD5:
 		fprintf(fp, "(HMAC_MD5)\n");
 		break;
