@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007, 2009-2012, 2015, 2016  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2007, 2009-2013  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1998-2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -231,6 +231,11 @@ dns_name_invalidate(dns_name_t *name);
  * \li	If the name had a dedicated buffer, that association is ended.
  */
 
+isc_boolean_t
+dns_name_isvalid(const dns_name_t *name);
+/*%<
+ * Check whether 'name' points to a valid dns_name
+ */
 
 /***
  *** Dedicated Buffers
@@ -989,6 +994,10 @@ dns_name_split(dns_name_t *name, unsigned int suffixlabels,
  *
  *\li	'suffix' is a valid name or NULL, and cannot be read-only.
  *
+ *\li	If non-NULL, 'prefix' and 'suffix' must have dedicated buffers.
+ *
+ *\li	'prefix' and 'suffix' cannot point to the same buffer.
+ *
  * Ensures:
  *
  *\li	On success:
@@ -1157,7 +1166,6 @@ dns_name_tostring(dns_name_t *source, char **target, isc_mem_t *mctx);
  * Returns:
  *
  *\li	ISC_R_SUCCESS
- *\li	ISC_R_NOMEMORY
  *
  *\li	Any error that dns_name_totext() can return.
  */
@@ -1280,24 +1288,6 @@ dns_name_destroy(void);
  * non-NULL argument prior to calling dns_name_destroy();
  */
 
-isc_boolean_t
-dns_name_isdnssd(const dns_name_t *owner);
-/*%<
- * Determine if the 'owner' is a DNS-SD prefix.
- */
-
-isc_boolean_t
-dns_name_isrfc1918(const dns_name_t *owner);
-/*%<
- * Determine if the 'name' is in the RFC 1918 reverse namespace.
- */
-
-isc_boolean_t
-dns_name_isula(const dns_name_t *owner);
-/*%<
- * Determine if the 'name' is in the ULA reverse namespace.
- */
-
 ISC_LANG_ENDDECLS
 
 /*
