@@ -1,5 +1,6 @@
 /*
- * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007, 2008, 2010, 2012-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1999-2002  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: rootns.c,v 1.40 2010/06/18 05:36:24 marka Exp $ */
+/* $Id: rootns.c,v 1.40.476.1 2012/02/07 00:44:14 each Exp $ */
 
 /*! \file */
 
@@ -60,20 +61,17 @@ static char root_ns[] =
 ".                       518400  IN      NS      M.ROOT-SERVERS.NET.\n"
 "A.ROOT-SERVERS.NET.     3600000 IN      A       198.41.0.4\n"
 "A.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:503:BA3E::2:30\n"
-"B.ROOT-SERVERS.NET.     3600000 IN      A       199.9.14.201\n"
-"B.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:200::b\n"
+"B.ROOT-SERVERS.NET.     3600000 IN      A       192.228.79.201\n"
 "C.ROOT-SERVERS.NET.     3600000 IN      A       192.33.4.12\n"
 "C.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:2::c\n"
 "D.ROOT-SERVERS.NET.     3600000 IN      A       199.7.91.13\n"
 "D.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:2d::d\n"
 "E.ROOT-SERVERS.NET.     3600000 IN      A       192.203.230.10\n"
-"E.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:a8::e\n"
 "F.ROOT-SERVERS.NET.     3600000 IN      A       192.5.5.241\n"
 "F.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:2F::F\n"
 "G.ROOT-SERVERS.NET.     3600000 IN      A       192.112.36.4\n"
-"G.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:12::d0d\n"
-"H.ROOT-SERVERS.NET.     3600000 IN      A       198.97.190.53\n"
-"H.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:1::53\n"
+"H.ROOT-SERVERS.NET.     3600000 IN      A       128.63.2.53\n"
+"H.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:1::803F:235\n"
 "I.ROOT-SERVERS.NET.     3600000 IN      A       192.36.148.17\n"
 "I.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:7fe::53\n"
 "J.ROOT-SERVERS.NET.     3600000 IN      A       192.58.128.30\n"
@@ -81,7 +79,7 @@ static char root_ns[] =
 "K.ROOT-SERVERS.NET.     3600000 IN      A       193.0.14.129\n"
 "K.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:7FD::1\n"
 "L.ROOT-SERVERS.NET.     3600000 IN      A       199.7.83.42\n"
-"L.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:500:9f::42\n"
+"L.ROOT-SERVERS.NET.     604800  IN      AAAA    2001:500:3::42\n"
 "M.ROOT-SERVERS.NET.     3600000 IN      A       202.12.27.33\n"
 "M.ROOT-SERVERS.NET.     3600000 IN      AAAA    2001:DC3::35\n";
 
@@ -130,7 +128,7 @@ check_node(dns_rdataset_t *rootns, dns_name_t *name,
 		case dns_rdatatype_ns:
 			if (dns_name_compare(name, dns_rootname) == 0)
 				break;
-			/* FALLTHROUGH */
+			/*FALLTHROUGH*/
 		default:
 			result = ISC_R_FAILURE;
 			goto cleanup;
@@ -213,18 +211,16 @@ dns_rootns_create(isc_mem_t *mctx, dns_rdataclass_t rdclass,
 	result = dns_db_create(mctx, "rbt", dns_rootname, dns_dbtype_zone,
 			       rdclass, 0, NULL, &db);
 	if (result != ISC_R_SUCCESS)
-		goto failure;
-
-	dns_rdatacallbacks_init(&callbacks);
+		return (result);
 
 	len = strlen(root_ns);
 	isc_buffer_init(&source, root_ns, len);
 	isc_buffer_add(&source, len);
 
-	result = dns_db_beginload(db, &callbacks.add,
-				  &callbacks.add_private);
+	dns_rdatacallbacks_init(&callbacks);
+	result = dns_db_beginload(db, &callbacks);
 	if (result != ISC_R_SUCCESS)
-		goto failure;
+		return (result);
 	if (filename != NULL) {
 		/*
 		 * Load the hints from the specified filename.
@@ -243,11 +239,11 @@ dns_rootns_create(isc_mem_t *mctx, dns_rdataclass_t rdclass,
 					       &callbacks, db->mctx);
 	} else
 		result = ISC_R_NOTFOUND;
-	eresult = dns_db_endload(db, &callbacks.add_private);
+	eresult = dns_db_endload(db, &callbacks);
 	if (result == ISC_R_SUCCESS || result == DNS_R_SEENINCLUDE)
 		result = eresult;
 	if (result != ISC_R_SUCCESS && result != DNS_R_SEENINCLUDE)
-		goto failure;
+		goto db_detach;
 	if (check_hints(db) != ISC_R_SUCCESS)
 		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
 			      DNS_LOGMODULE_HINTS, ISC_LOG_WARNING,
@@ -256,14 +252,8 @@ dns_rootns_create(isc_mem_t *mctx, dns_rdataclass_t rdclass,
 	*target = db;
 	return (ISC_R_SUCCESS);
 
- failure:
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_HINTS,
-		      ISC_LOG_ERROR, "could not configure root hints from "
-		      "'%s': %s", (filename != NULL) ? filename : "<BUILT-IN>",
-		      isc_result_totext(result));
-
-	if (db != NULL)
-		dns_db_detach(&db);
+ db_detach:
+	dns_db_detach(&db);
 
 	return (result);
 }
